open Graphics
open Graph
open Gio
open Printf

let draw_node n x y color =
  let (w,h) = text_size (string_of_int n)
  in let r = int_of_float (ceil ((sqrt (((float_of_int w) /. 2.) ** 2. +.
					  ((float_of_int h) /. 2.) ** 2.)) +. 1.))
  in 
    set_color color;
    fill_circle x y r;
    set_color foreground;
    moveto (x - w / 2) (y - h / 2);
    draw_string (string_of_int n);
    draw_circle x y r

let draw_nodes graph colors =
  Array.iteri
    (fun i (x,y) -> draw_node i x y colors.(i))
    (pos graph)

let my_line (x1,y1) (x2,y2) =
  moveto x1 y1;
  lineto x2 y2 

let draw_edges graph =
  Array.iteri
    (fun i nbs ->
       Array.iter
	 (fun v ->
	    if i < v then my_line (pos graph).(i) (pos graph).(v))
	 nbs)
    (tab graph)

let ps_line c_out (x1,y1) (x2,y2) =
  fprintf c_out "0 setgray %d %d moveto %d %d lineto\n" x1 y1 x2 y2

let draw_edges_ps c_out graph =
  Array.iteri
    (fun i nbs ->
       Array.iter
	 (fun v ->
	    if i < v then ps_line c_out (pos graph).(i) (pos graph).(v))
	 nbs)
    (tab graph)

let ps_node c_out n x y =
  fprintf c_out "%d %d (%d) drawnode\n" x y n

let draw_nodes_ps c_out graph =
  Array.iteri
    (fun i (x,y) -> ps_node c_out i x y)
    (pos graph)

let draw graph colors =
  clear_graph ();
  draw_edges graph;
  draw_nodes graph colors

let draw_ps graph filename =
  let header_in = open_in "header"
  and file_out = open_out filename
  in
    begin
      fprintf file_out "%sBoundingBox: %d %d %d %d\n\n" "%%" 0 0 (size_x ()) (size_y ());
      (try
	 while true do 
	   (output_string file_out (input_line header_in);
	    output_string file_out "\n")
	 done
       with 
	   End_of_file -> close_in header_in);
      draw_edges_ps file_out graph;
      fprintf file_out "stroke\n\n";
      draw_nodes_ps file_out graph;
      fprintf file_out "\nshowpage\n";
      close_out file_out
    end
    
	  

let redraw graph colors =
  recomputepos graph;
  draw graph colors
